// The CERN-OHL-P v2 License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// This source describes Open Hardware and is licensed under the CERN-OHL-P v2.
// You may redistribute and modify this documentation and make products using it under the terms of
// the CERN-OHL-P v2 (https:/cern.ch/cern-ohl). This documentation is distributed WITHOUT ANY
// EXPRESS OR IMPLIED WARRANTY, INCLUDING OF MERCHANTABILITY, SATISFACTORY QUALITY AND FITNESS FOR A
// PARTICULAR PURPOSE. Please see the CERN-OHL-P v2 for applicable conditions.
//
// Revisions:
// Rev.1.0		2025/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
module PrimroseSoC # (
		parameter [31:0] RTL_VERSION = "100A",
		parameter [31:0] RTL_DATE = {16'd2024, 8'd4, 8'd16},
		parameter CONNECTOR_TYPE = "Raspberry",
//		parameter CONNECTOR_TYPE = "Arduino",
//		parameter I2C_PIN = "I2C",
		parameter I2C_PIN = "GPIO",
//		parameter SPI_PIN = "SPI",
		parameter SPI_PIN = "GPIO",
//		parameter SPI_CE1_PIN = "SPI",		// Only Raspberry
		parameter SPI_CE1_PIN = "GPIO",		// Only Raspberry
//		parameter SPI_CE0_PIN = "SPI",		// Only Raspberry
		parameter SPI_CE0_PIN = "GPIO",		// Only Raspberry
//		parameter UART_PIN = "UART",
		parameter UART_PIN = "GPIO",
		parameter APB3_ADDR_WIDTH = 16,
		parameter APB3_DATA_WIDTH = 32,
		parameter LVDS_TEST_NUMS = 5,
		parameter LVDS_WIDTH = 8,
		parameter SLED_ADDR_WIDTH = 2,								// Max 7bit	128LEDs
		parameter SLED_DATA_WIDTH = 24								// Fixed 24bit
	) (
		// System signals
		input MAIN_CLK,
		input PLL_LOCKED,

		// Power monitor and control
		input PWR_SW_N_IN,
		input VIN_MON_N_IN,
		input VBUS_MON_N_IN,
		output PWR_CNT_OUT,
		output USB_POW_DIS_OUT,
		output ISET_1R0A_N_OUT,
		output ISET_1R5A_N_OUT,

		// USB TYPE-C
		input TYPEC_SWMONI_IN,
		input [1:0] TYPEC_TCC_IN,

		// USB VCP CP2102N
		output USB_VCP_TXD_OUT,
		input USB_VCP_RXD_IN,
		output USB_VCP_RTS_OUT,
		input USB_VCP_CTS_IN,
		input [1:0] USB_CHR_IN,
		input USB_CHREN_IN,
		input USB_SUSPEND_IN,

		// SD Card
		output SD_CS_N_OUT,				// SD CS (DATA3/CS)
		input SD_MOSI_IN,
		output SD_MOSI_OUT,				// SD DI (CMD/DI)
		output SD_MOSI_OE,
		output SD_SCLK_OUT,				// SD SCK (CLK/SCLK)
		input SD_MISO_IN,				// SD DO (DATA0/DO)
		output SD_MISO_OUT,
		output SD_MISO_OE,
		input SD_DET_N_IN,

		// Flash, ADC, DAC
		output ADDA_FLASH_SCK_OUT,
		input ADDA_FLASH_MOSI_IN,
		output ADDA_FLASH_MOSI_OUT,
		output ADDA_FLASH_MOSI_OE,
		input ADDA_FLASH_MISO_IN,
		output ADDA_FLASH_MISO_OUT,
		output ADDA_FLASH_MISO_OE,
		output FLASH_CS_N_OUT,
		output ADC_CS_N_OUT,
		output DAC_CS_N_OUT,
		output AREF_EXT_SEL_N_OUT,

		// DIPSW, LED, Power LED, Serial RGB LED 
		input [3:0] DIPSW_N_IN,
		output [3:0] LED_N_OUT,
		output PWR_LED_R_N_OUT,
		output PWR_LED_G_N_OUT,
		output PWR_LED_B_N_OUT,
		output SER_LED_OUT,

		// Raspberry PI3, Arduino Zero compatible connector input
		input GPIO0_ID_SD_PWM_IO5_IN,
		input GPIO1_ID_SC_PWM_IO6_IN,
		input GPIO2_SDA_IO20_IN,
		input GPIO3_SCL_IO21_IN,
		input GPIO4_IN,
		input GPIO5_IO4_IN,
		input GPIO6_IO2_IN,
		input GPIO7_CE1_PMM_IO9_IN,
		input GPIO8_CE0_SS_PMM_IO10_IN,
		input GPIO9_MISO_IO12_IN,
		input GPIO10_MOSI_PWM_IO11_IN,
		input GPIO11_SCLK_SCK_IO13_IN,
		input GPIO12_PWM0_PWM_IO3_IN,
		input GPIO13_PWM1_IO19_IN,
		input GPIO14_TXD0_TX_IO1_IN,
		input GPIO15_RXD0_RX_IO0_IN,
		input GPIO16_IO18_IN,
		input GPIO17_IN,
		input GPIO18_IN,
		input GPIO19_PCM_FS_IO17_IN,
		input GPIO20_PCM_DIN_IO16_IN,
		input GPIO21_PCM_DOUT_IO14_IN,
		input GPIO22_IN,
		input GPIO23_IN,
		input GPIO24_IO8_IN,
		input GPIO25_IO7_IN,
		input GPIO26_IO15_IN,
		input GPIO27_IN,

		// Raspberry PI / Arduino compatible connector output
		output GPIO0_ID_SD_PWM_IO5_OUT,
		output GPIO1_ID_SC_PWM_IO6_OUT,
		output GPIO2_SDA_IO20_OUT,
		output GPIO3_SCL_IO21_OUT,
		output GPIO4_OUT,
		output GPIO5_IO4_OUT,
		output GPIO6_IO2_OUT,
		output GPIO7_CE1_PMM_IO9_OUT,
		output GPIO8_CE0_SS_PMM_IO10_OUT,
		output GPIO9_MISO_IO12_OUT,
		output GPIO10_MOSI_PWM_IO11_OUT,
		output GPIO11_SCLK_SCK_IO13_OUT,
		output GPIO12_PWM0_PWM_IO3_OUT,
		output GPIO13_PWM1_IO19_OUT,
		output GPIO14_TXD0_TX_IO1_OUT,
		output GPIO15_RXD0_RX_IO0_OUT,
		output GPIO16_IO18_OUT,
		output GPIO17_OUT,
		output GPIO18_OUT,
		output GPIO19_PCM_FS_IO17_OUT,
		output GPIO20_PCM_DIN_IO16_OUT,
		output GPIO21_PCM_DOUT_IO14_OUT,
		output GPIO22_OUT,
		output GPIO23_OUT,
		output GPIO24_IO8_OUT,
		output GPIO25_IO7_OUT,
		output GPIO26_IO15_OUT,
		output GPIO27_OUT,

	// Raspberry PI / Arduino compatible connector output enable
		output GPIO0_ID_SD_PWM_IO5_OE,
		output GPIO1_ID_SC_PWM_IO6_OE,
		output GPIO2_SDA_IO20_OE,
		output GPIO3_SCL_IO21_OE,
		output GPIO4_OE,
		output GPIO5_IO4_OE,
		output GPIO6_IO2_OE,
		output GPIO7_CE1_PMM_IO9_OE,
		output GPIO8_CE0_SS_PMM_IO10_OE,
		output GPIO9_MISO_IO12_OE,
		output GPIO10_MOSI_PWM_IO11_OE,
		output GPIO11_SCLK_SCK_IO13_OE,
		output GPIO12_PWM0_PWM_IO3_OE,
		output GPIO13_PWM1_IO19_OE,
		output GPIO14_TXD0_TX_IO1_OE,
		output GPIO15_RXD0_RX_IO0_OE,
		output GPIO16_IO18_OE,
		output GPIO17_OE,
		output GPIO18_OE,
		output GPIO19_PCM_FS_IO17_OE,
		output GPIO20_PCM_DIN_IO16_OE,
		output GPIO21_PCM_DOUT_IO14_OE,
		output GPIO22_OE,
		output GPIO23_OE,
		output GPIO24_IO8_OE,
		output GPIO25_IO7_OE,
		output GPIO26_IO15_OE,
		output GPIO27_OE,

		// LVDS TX
		output LVDS_TXPLL_RST_N,
		input LVDS_TXPLL_LOCKED,
		input LVDS_TXCLK,
		output [LVDS_WIDTH-1:0] LVDS_TX_DATA0,
		output [LVDS_WIDTH-1:0] LVDS_TX_DATA1,
		output [LVDS_WIDTH-1:0] LVDS_TX_DATA2,
		output [LVDS_WIDTH-1:0] LVDS_TX_DATA3,
//		output [DATA_WIDTH-1:0] LVDS_TX_DATA4,
		output LVDS_TX_DATA4,                       // Disable serialization
//		output [LVDS_WIDTH-1:0] LVDS_TX_DATA5,		// Assign to TX FAST CLOCK

		// LVDS RX
		output LVDS_RXPLL_RST_N,
		input LVDS_RXPLL_LOCKED,
		input LVDS_RXCLK,
		input [LVDS_WIDTH-1:0] LVDS_RX_DATA0,
		input [LVDS_WIDTH-1:0] LVDS_RX_DATA1,
		input [LVDS_WIDTH-1:0] LVDS_RX_DATA2,
		input [LVDS_WIDTH-1:0] LVDS_RX_DATA3,
		input LVDS_RX_DATA4,                        // Disable serialization
		input LVDS_RX_DATA5,                        // Disable serialization and invert polarity

		// JTAG
		input jtag_inst1_TCK,
		input jtag_inst1_TDI,
		output jtag_inst1_TDO,
		input jtag_inst1_SEL,
		input jtag_inst1_CAPTURE,
		input jtag_inst1_SHIFT
	);

	`define BIDIR_CONNECT(PORT, WIRE, INDEX)		assign WIRE``_read``INDEX = PORT``_IN;			\
													assign PORT``_OUT = WIRE``_write``INDEX;		\
													assign PORT``_OE = WIRE``_writeEnable``INDEX
	`define BIDIR_READ_CONNECT(PORT, WIRE, INDEX)	assign WIRE``INDEX = PORT``_IN;					\
													assign PORT``_OUT = 1'b0;						\
													assign PORT``_OE = 1'b0
	`define BIDIR_WRITE_CONNECT(PORT, WIRE, INDEX)	assign PORT``_OUT = WIRE``INDEX;				\
													assign PORT``_OE = 1'b1

	wire io_asyncReset;
	wire io_systemReset;

	wire [APB3_ADDR_WIDTH-1:0] io_apbSlave_0_PADDR;
	wire [0:0] io_apbSlave_0_PSEL;
	wire io_apbSlave_0_PENABLE;
	wire io_apbSlave_0_PREADY;
	wire io_apbSlave_0_PWRITE;
	wire [APB3_DATA_WIDTH-1:0] io_apbSlave_0_PWDATA;
	wire [APB3_DATA_WIDTH-1:0] io_apbSlave_0_PRDATA;
	wire io_apbSlave_0_PSLVERROR;

	wire [LVDS_TEST_NUMS-1:0] lvds_test_run;
	wire [LVDS_TEST_NUMS-1:0] lvds_test_rx;
	wire [LVDS_TEST_NUMS-1:0] lvds_test_err;

	wire system_spi_0_io_data_0_read;
	wire system_spi_0_io_data_0_write;
	wire system_spi_0_io_data_0_writeEnable;
	wire system_spi_0_io_data_1_read;
	wire system_spi_0_io_data_1_write;
	wire system_spi_0_io_data_1_writeEnable;
	wire system_spi_0_io_sclk_write;
	wire [0:0] system_spi_0_io_ss;

	wire system_spi_1_io_data_0_read;
	wire system_spi_1_io_data_0_write;
	wire system_spi_1_io_data_0_writeEnable;
	wire system_spi_1_io_data_1_read;
	wire system_spi_1_io_data_1_write;
	wire system_spi_1_io_data_1_writeEnable;
	wire system_spi_1_io_sclk_write;
	wire [2:0] system_spi_1_io_ss;

	wire system_spi_2_io_data_0_read;
	wire system_spi_2_io_data_0_write;
	wire system_spi_2_io_data_0_writeEnable;
	wire system_spi_2_io_data_1_read;
	wire system_spi_2_io_data_1_write;
	wire system_spi_2_io_data_1_writeEnable;
	wire system_spi_2_io_sclk_write;
	wire [1:0] system_spi_2_io_ss;

	wire system_i2c_0_io_scl_read;
	wire system_i2c_0_io_scl_write;
	wire system_i2c_0_io_scl_writeEnable = !system_i2c_0_io_scl_write;
	wire system_i2c_0_io_sda_read;
	wire system_i2c_0_io_sda_write;
	wire system_i2c_0_io_sda_writeEnable = !system_i2c_0_io_sda_write;

	wire system_uart_0_io_txd;
	wire system_uart_0_io_rxd;

	wire system_uart_1_io_rxd;
	wire system_uart_1_io_txd;

	wire [15:0] system_gpio_0_io_read;
	wire [15:0] system_gpio_0_io_write;
	wire [15:0] system_gpio_0_io_writeEnable;
	wire [15:0] system_gpio_1_io_read;
	wire [15:0] system_gpio_1_io_write;
	wire [15:0] system_gpio_1_io_writeEnable;

	wire userInterruptA;

	wire [SLED_ADDR_WIDTH-1:0] ser_led_addr;
	wire [SLED_DATA_WIDTH-1:0] ser_led_data;
	wire ser_led_wr;
	wire ser_led_set;
	wire ser_led_busy;

	assign io_asyncReset = ~PLL_LOCKED;

	// USB VCP CP2102N
	assign USB_VCP_TXD_OUT = system_uart_0_io_txd;
	assign system_uart_0_io_rxd = USB_VCP_RXD_IN;

	// SD Card
	`BIDIR_CONNECT(SD_MOSI, system_spi_0_io_data_0, );
	`BIDIR_CONNECT(SD_MISO, system_spi_0_io_data_1, );
	assign SD_SCLK_OUT = system_spi_0_io_sclk_write;
	assign SD_CS_N_OUT = system_spi_0_io_ss[0];

	// Flash, ADC, DAC
	`BIDIR_CONNECT(ADDA_FLASH_MOSI, system_spi_1_io_data_0, );
	`BIDIR_CONNECT(ADDA_FLASH_MISO, system_spi_1_io_data_1, );
	assign ADDA_FLASH_SCK_OUT = system_spi_1_io_sclk_write;
	assign FLASH_CS_N_OUT = system_spi_1_io_ss[0];
	assign ADC_CS_N_OUT = system_spi_1_io_ss[1];
	assign DAC_CS_N_OUT = system_spi_1_io_ss[2];

	if (CONNECTOR_TYPE == "Raspberry") begin
		`BIDIR_CONNECT(GPIO0_ID_SD_PWM_IO5, system_gpio_0_io, [0]);
		`BIDIR_CONNECT(GPIO1_ID_SC_PWM_IO6, system_gpio_0_io, [1]);
		if (I2C_PIN == "I2C") begin
			`BIDIR_CONNECT(GPIO2_SDA_IO20, system_i2c_0_io_sda, );
			`BIDIR_CONNECT(GPIO3_SCL_IO21, system_i2c_0_io_scl, );
		end
		else if (I2C_PIN == "GPIO") begin
			`BIDIR_CONNECT(GPIO2_SDA_IO20, system_gpio_0_io, [2]);
			`BIDIR_CONNECT(GPIO3_SCL_IO21, system_gpio_0_io, [3]);
		end
		`BIDIR_CONNECT(GPIO4, system_gpio_0_io, [4]);
		`BIDIR_CONNECT(GPIO5_IO4, system_gpio_0_io, [5]);
		`BIDIR_CONNECT(GPIO6_IO2, system_gpio_0_io, [6]);
		if (SPI_PIN == "SPI") begin
			if (SPI_CE1_PIN == "SPI") begin
			`BIDIR_WRITE_CONNECT(GPIO7_CE1_PMM_IO9, system_spi_2_io_ss, [1]);
			end
			else if (SPI_CE1_PIN == "GPIO") begin
			`BIDIR_CONNECT(GPIO7_CE1_PMM_IO9, system_gpio_0_io, [7]);
			end
			if (SPI_CE0_PIN == "SPI") begin
			`BIDIR_WRITE_CONNECT(GPIO8_CE0_SS_PMM_IO10, system_spi_2_io_ss, [0]);
			end
			else if (SPI_CE1_PIN == "GPIO") begin
			`BIDIR_CONNECT(GPIO8_CE0_SS_PMM_IO10, system_gpio_0_io, [8]);
			end
			`BIDIR_CONNECT(GPIO9_MISO_IO12, system_spi_2_io_data_1, );
			`BIDIR_CONNECT(GPIO10_MOSI_PWM_IO11, system_spi_2_io_data_0, );
			`BIDIR_WRITE_CONNECT(GPIO11_SCLK_SCK_IO13, system_spi_2_io_sclk_write, );
		end
		else if (I2C_PIN == "GPIO") begin
			`BIDIR_CONNECT(GPIO7_CE1_PMM_IO9, system_gpio_0_io, [7]);
			`BIDIR_CONNECT(GPIO8_CE0_SS_PMM_IO10, system_gpio_0_io, [8]);
			`BIDIR_CONNECT(GPIO9_MISO_IO12, system_gpio_0_io, [9]);
			`BIDIR_CONNECT(GPIO10_MOSI_PWM_IO11, system_gpio_0_io, [10]);
			`BIDIR_CONNECT(GPIO11_SCLK_SCK_IO13, system_gpio_0_io, [11]);
		end
		`BIDIR_CONNECT(GPIO12_PWM0_PWM_IO3, system_gpio_0_io, [12]);
		`BIDIR_CONNECT(GPIO13_PWM1_IO19, system_gpio_0_io, [13]);
		if (UART_PIN == "UART") begin
			`BIDIR_WRITE_CONNECT(GPIO14_TXD0_TX_IO1, system_uart_1_io_txd, );
			`BIDIR_READ_CONNECT(GPIO15_RXD0_RX_IO0, system_uart_1_io_rxd, );
		end
		else if (UART_PIN == "GPIO") begin
			`BIDIR_CONNECT(GPIO14_TXD0_TX_IO1, system_gpio_0_io, [14]);
			`BIDIR_CONNECT(GPIO15_RXD0_RX_IO0, system_gpio_0_io, [15]);
		end
		`BIDIR_CONNECT(GPIO16_IO18, system_gpio_1_io, [0]);
		`BIDIR_CONNECT(GPIO17, system_gpio_1_io, [1]);
		`BIDIR_CONNECT(GPIO18, system_gpio_1_io, [2]);
		`BIDIR_CONNECT(GPIO19_PCM_FS_IO17, system_gpio_1_io, [3]);
		`BIDIR_CONNECT(GPIO20_PCM_DIN_IO16, system_gpio_1_io, [4]);
		`BIDIR_CONNECT(GPIO21_PCM_DOUT_IO14, system_gpio_1_io, [5]);
		`BIDIR_CONNECT(GPIO22, system_gpio_1_io, [6]);
		`BIDIR_CONNECT(GPIO23, system_gpio_1_io, [7]);
		`BIDIR_CONNECT(GPIO24_IO8, system_gpio_1_io, [8]);
		`BIDIR_CONNECT(GPIO25_IO7, system_gpio_1_io, [9]);
		`BIDIR_CONNECT(GPIO26_IO15, system_gpio_1_io, [10]);
		`BIDIR_CONNECT(GPIO27, system_gpio_1_io, [11]);
	end
	else if (CONNECTOR_TYPE == "Arduino") begin
		if (UART_PIN == "UART") begin
		`BIDIR_READ_CONNECT(GPIO15_RXD0_RX_IO0, system_uart_1_io_rxd, );
		`BIDIR_WRITE_CONNECT(GPIO14_TXD0_TX_IO1, system_uart_1_io_txd, );
		end
		else if (UART_PIN == "GPIO") begin
		`BIDIR_CONNECT(GPIO15_RXD0_RX_IO0, system_gpio_0_io, [0]);
		`BIDIR_CONNECT(GPIO14_TXD0_TX_IO1, system_gpio_0_io, [1]);
		end
		`BIDIR_CONNECT(GPIO6_IO2, system_gpio_0_io, [2]);
		`BIDIR_CONNECT(GPIO12_PWM0_PWM_IO3, system_gpio_0_io, [3]);
		`BIDIR_CONNECT(GPIO5_IO4, system_gpio_0_io, [4]);
		`BIDIR_CONNECT(GPIO0_ID_SD_PWM_IO5, system_gpio_0_io, [5]);
		`BIDIR_CONNECT(GPIO1_ID_SC_PWM_IO6, system_gpio_0_io, [6]);
		`BIDIR_CONNECT(GPIO25_IO7, system_gpio_0_io, [7]);
		`BIDIR_CONNECT(GPIO24_IO8, system_gpio_0_io, [8]);
		`BIDIR_CONNECT(GPIO7_CE1_PMM_IO9, system_gpio_0_io, [9]);
		if (SPI_PIN == "SPI") begin
		`BIDIR_WRITE_CONNECT(GPIO8_CE0_SS_PMM_IO10, system_spi_2_io_ss, [0]);
		`BIDIR_CONNECT(GPIO10_MOSI_PWM_IO11, system_spi_2_io_data_0, );
		`BIDIR_CONNECT(GPIO9_MISO_IO12, system_spi_2_io_data_1, );
		`BIDIR_WRITE_CONNECT(GPIO11_SCLK_SCK_IO13, system_spi_2_io_sclk_write, );
		end
		else if (I2C_PIN == "GPIO") begin
		`BIDIR_CONNECT(GPIO8_CE0_SS_PMM_IO10, system_gpio_0_io, [10]);
		`BIDIR_CONNECT(GPIO10_MOSI_PWM_IO11, system_gpio_0_io, [11]);
		`BIDIR_CONNECT(GPIO9_MISO_IO12, system_gpio_0_io, [12]);
		`BIDIR_CONNECT(GPIO11_SCLK_SCK_IO13, system_gpio_0_io, [13]);
		end
		`BIDIR_CONNECT(GPIO21_PCM_DOUT_IO14, system_gpio_0_io, [14]);
		`BIDIR_CONNECT(GPIO26_IO15, system_gpio_0_io, [15]);
		`BIDIR_CONNECT(GPIO20_PCM_DIN_IO16, system_gpio_1_io, [0]);
		`BIDIR_CONNECT(GPIO19_PCM_FS_IO17, system_gpio_1_io, [1]);
		`BIDIR_CONNECT(GPIO16_IO18, system_gpio_1_io, [2]);
		`BIDIR_CONNECT(GPIO13_PWM1_IO19, system_gpio_1_io, [3]);
		if (I2C_PIN == "I2C") begin
		`BIDIR_CONNECT(GPIO2_SDA_IO20, system_i2c_0_io_sda, );
		`BIDIR_CONNECT(GPIO3_SCL_IO21, system_i2c_0_io_scl, );
		end
		else if (I2C_PIN == "GPIO") begin
		`BIDIR_CONNECT(GPIO2_SDA_IO20, system_gpio_1_io, [4]);
		`BIDIR_CONNECT(GPIO3_SCL_IO21, system_gpio_1_io, [5]);
		end
	end

	assign system_gpio_1_io_read[15:12] = ~DIPSW_N_IN;
	assign LED_N_OUT = ~system_gpio_1_io_write[15:12];

	apb3_slave # (
		.ADDR_WIDTH(APB3_ADDR_WIDTH),
		.DATA_WIDTH(APB3_DATA_WIDTH),
		.LVDS_TEST_NUMS(LVDS_TEST_NUMS),
		.ADDR_WIDTH(SLED_ADDR_WIDTH),
		.DATA_WIDTH(SLED_DATA_WIDTH)
	) apb3_slave_u (
		.CLOCK(MAIN_CLK),
		.RESET_N(~io_systemReset),

		.INTERRUPT(userInterruptA),

		// APB3
		.PADDR(io_apbSlave_0_PADDR),
		.PSEL(io_apbSlave_0_PSEL[0]),
		.PENABLE(io_apbSlave_0_PENABLE),
		.PREADY(io_apbSlave_0_PREADY),
		.PWRITE(io_apbSlave_0_PWRITE),
		.PWDATA(io_apbSlave_0_PWDATA),
		.PRDATA(io_apbSlave_0_PRDATA),
		.PSLVERROR(io_apbSlave_0_PSLVERROR),

		// System information
		.RTL_VERSION_IN(RTL_VERSION),
		.RTL_DATE_IN(RTL_DATE),

		// Power monitor and control
		.PWR_SW_N_IN(PWR_SW_N_IN),
		.VIN_MON_N_IN(VIN_MON_N_IN),
		.VBUS_MON_N_IN(VBUS_MON_N_IN),
		.PWR_CNT_OUT(PWR_CNT_OUT),
		.USB_POW_DIS_OUT(USB_POW_DIS_OUT),
		.ISET_1R0A_N_OUT(ISET_1R0A_N_OUT),
		.ISET_1R5A_N_OUT(ISET_1R5A_N_OUT),
	
		// USB TYPE-C
		.TYPEC_SWMONI_IN(TYPEC_SWMONI_IN),
		.TYPEC_TCC_IN(TYPEC_TCC_IN),

		// USB VCP CP2102N
		.USB_VCP_RTS_OUT(USB_VCP_RTS_OUT),
		.USB_VCP_CTS_IN(USB_VCP_CTS_IN),
		.USB_CHR_IN(USB_CHR_IN),
		.USB_CHREN_IN(USB_CHREN_IN),
		.USB_SUSPEND_IN(USB_SUSPEND_IN),

		// SD Card
		.SD_DET_N_IN(SD_DET_N_IN),

		// Flash, ADC, DAC
		.AREF_EXT_SEL_N_OUT(AREF_EXT_SEL_N_OUT),

		// LED 
		.PWR_LED_R_N_OUT(PWR_LED_R_N_OUT),
		.PWR_LED_G_N_OUT(PWR_LED_G_N_OUT),
		.PWR_LED_B_N_OUT(PWR_LED_B_N_OUT),
		.SER_LED_ADDR_OUT(ser_led_addr),
		.SER_LED_DATA_OUT(ser_led_data),
		.SER_LED_WRITE_OUT(ser_led_wr),
		.SER_LED_SET_OUT(ser_led_set),
		.SER_LED_BUSY_IN(ser_led_busy),

		// LVDS
		.LVDS_TXPLL_RST_N_OUT(LVDS_TXPLL_RST_N),
		.LVDS_TXPLL_LOCKED_IN(LVDS_TXPLL_LOCKED),
		.LVDS_RXPLL_RST_N_OUT(LVDS_RXPLL_RST_N),
		.LVDS_RXPLL_LOCKED_IN(LVDS_RXPLL_LOCKED),
		.LVDS_TEST_RUN_OUT(lvds_test_run),
		.LVDS_TEST_RX_IN(lvds_test_rx),
		.LVDS_TEST_ERR_IN(lvds_test_err)
	);

	lvds_test # (
		.TEST_NUMS(LVDS_TEST_NUMS),
		.DATA_WIDTH(LVDS_WIDTH)
	) lvds_test_u (
		.TX_CLK(LVDS_TXCLK),
		.TX_PLL_LOCKED(LVDS_TXPLL_LOCKED),
		.RX_CLK(LVDS_RXCLK),
		.RX_PLL_LOCKED(LVDS_RXPLL_LOCKED),
		.IN_RUN(lvds_test_run),
		.LVDS_RX_DATA0(LVDS_RX_DATA0),
		.LVDS_RX_DATA1(LVDS_RX_DATA1),
		.LVDS_RX_DATA2(LVDS_RX_DATA2),
		.LVDS_RX_DATA3(LVDS_RX_DATA3),
		.LVDS_RX_DATA4(LVDS_RX_DATA4),
		.LVDS_RX_DATA5(LVDS_RX_DATA5),
		.LVDS_TX_DATA0(LVDS_TX_DATA0),
		.LVDS_TX_DATA1(LVDS_TX_DATA1),
		.LVDS_TX_DATA2(LVDS_TX_DATA2),
		.LVDS_TX_DATA3(LVDS_TX_DATA3),
		.LVDS_TX_DATA4(LVDS_TX_DATA4),
//		.LVDS_TX_DATA5(LVDS_TX_DATA5),	// Assign to TX FAST CLOCK
		.OUT_RX_EXISTS(lvds_test_rx),
		.OUT_ERROR(lvds_test_err)
	);

	sled_drv # (                // IN-PI42TASPRPGPB parameters
		.CLOCK_POL(1),
		.RESET_POL(1),
		.ARESET(1),
		.T_WIDTH(31),           // 1.24us * 25MHz = 31
		.T0H_WIDTH(8),			// 0.32us * 25MHz = 8
		.T1H_WIDTH(22),			// 0.88us * 25MHz = 22
		.T_RESET_WIDTH(5025),	// 201us * 25MHz = 5025
		.LED_NUMS(3),
		.ADDR_WIDTH(SLED_ADDR_WIDTH),
		.DATA_WIDTH(SLED_DATA_WIDTH)
	) sled_drv_u (
		.CLOCK(MAIN_CLK),
		.RESET(io_systemReset),
		.IN_ADDR(ser_led_addr),
		.IN_DATA(ser_led_data),
		.IN_WRITE(ser_led_wr),
		.IN_LED_SET(ser_led_set),
		.OUT_BUSY(ser_led_busy),
		.OUT_LED_DATA(SER_LED_OUT)
	);

	SapphireSoC SapphireSoC_u (
		.io_systemClk(MAIN_CLK),
		.io_asyncReset(io_asyncReset),
		.io_systemReset(io_systemReset),

		.io_apbSlave_0_PADDR(io_apbSlave_0_PADDR),
		.io_apbSlave_0_PENABLE(io_apbSlave_0_PENABLE),
		.io_apbSlave_0_PRDATA(io_apbSlave_0_PRDATA),
		.io_apbSlave_0_PREADY(io_apbSlave_0_PREADY),
		.io_apbSlave_0_PSEL(io_apbSlave_0_PSEL),
		.io_apbSlave_0_PSLVERROR(io_apbSlave_0_PSLVERROR),
		.io_apbSlave_0_PWDATA(io_apbSlave_0_PWDATA),
		.io_apbSlave_0_PWRITE(io_apbSlave_0_PWRITE),

		.system_spi_0_io_data_0_read(system_spi_0_io_data_0_read),
		.system_spi_0_io_data_0_write(system_spi_0_io_data_0_write),
		.system_spi_0_io_data_0_writeEnable(system_spi_0_io_data_0_writeEnable),
		.system_spi_0_io_data_1_read(system_spi_0_io_data_1_read),
		.system_spi_0_io_data_1_write(system_spi_0_io_data_1_write),
		.system_spi_0_io_data_1_writeEnable(system_spi_0_io_data_1_writeEnable),
		.system_spi_0_io_data_2_read(),
		.system_spi_0_io_data_2_write(),
		.system_spi_0_io_data_2_writeEnable(),
		.system_spi_0_io_data_3_read(),
		.system_spi_0_io_data_3_write(),
		.system_spi_0_io_data_3_writeEnable(),
		.system_spi_0_io_sclk_write(system_spi_0_io_sclk_write),
		.system_spi_0_io_ss(system_spi_0_io_ss),

		.system_spi_1_io_data_0_read(system_spi_1_io_data_0_read),
		.system_spi_1_io_data_0_write(system_spi_1_io_data_0_write),
		.system_spi_1_io_data_0_writeEnable(system_spi_1_io_data_0_writeEnable),
		.system_spi_1_io_data_1_read(system_spi_1_io_data_1_read),
		.system_spi_1_io_data_1_write(system_spi_1_io_data_1_write),
		.system_spi_1_io_data_1_writeEnable(system_spi_1_io_data_1_writeEnable),
		.system_spi_1_io_data_2_read(),
		.system_spi_1_io_data_2_write(),
		.system_spi_1_io_data_2_writeEnable(),
		.system_spi_1_io_data_3_read(),
		.system_spi_1_io_data_3_write(),
		.system_spi_1_io_data_3_writeEnable(),
		.system_spi_1_io_sclk_write(system_spi_1_io_sclk_write),
		.system_spi_1_io_ss(system_spi_1_io_ss),

		.system_spi_2_io_data_0_read(system_spi_2_io_data_0_read),
		.system_spi_2_io_data_0_write(system_spi_2_io_data_0_write),
		.system_spi_2_io_data_0_writeEnable(system_spi_2_io_data_0_writeEnable),
		.system_spi_2_io_data_1_read(system_spi_2_io_data_1_read),
		.system_spi_2_io_data_1_write(system_spi_2_io_data_1_write),
		.system_spi_2_io_data_1_writeEnable(system_spi_2_io_data_1_writeEnable),
		.system_spi_2_io_data_2_read(),
		.system_spi_2_io_data_2_write(),
		.system_spi_2_io_data_2_writeEnable(),
		.system_spi_2_io_data_3_read(),
		.system_spi_2_io_data_3_write(),
		.system_spi_2_io_data_3_writeEnable(),
		.system_spi_2_io_sclk_write(system_spi_2_io_sclk_write),
		.system_spi_2_io_ss(system_spi_2_io_ss),

		.system_uart_0_io_txd(system_uart_0_io_txd),
		.system_uart_0_io_rxd(system_uart_0_io_rxd),
		.system_uart_1_io_rxd(system_uart_1_io_rxd),
		.system_uart_1_io_txd(system_uart_1_io_txd),

		.system_i2c_0_io_scl_read(system_i2c_0_io_scl_read),
		.system_i2c_0_io_scl_write(system_i2c_0_io_scl_write),
		.system_i2c_0_io_sda_read(system_i2c_0_io_sda_read),
		.system_i2c_0_io_sda_write(system_i2c_0_io_sda_write),

		.system_gpio_0_io_read(system_gpio_0_io_read),
		.system_gpio_0_io_write(system_gpio_0_io_write),
		.system_gpio_0_io_writeEnable(system_gpio_0_io_writeEnable),
		.system_gpio_1_io_read(system_gpio_1_io_read),
		.system_gpio_1_io_write(system_gpio_1_io_write),
		.system_gpio_1_io_writeEnable(system_gpio_1_io_writeEnable),

		.userInterruptA(userInterruptA),

		.jtagCtrl_enable(jtag_inst1_SEL),
		.jtagCtrl_tdi(jtag_inst1_TDI),
		.jtagCtrl_capture(jtag_inst1_CAPTURE),
		.jtagCtrl_shift(jtag_inst1_SHIFT),
		.jtagCtrl_update(),
		.jtagCtrl_reset(),
		.jtagCtrl_tdo(jtag_inst1_TDO),
		.jtagCtrl_tck(jtag_inst1_TCK)
	);

endmodule
