// The CERN-OHL-P v2 License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// This source describes Open Hardware and is licensed under the CERN-OHL-P v2.
// You may redistribute and modify this documentation and make products using it under the terms of
// the CERN-OHL-P v2 (https:/cern.ch/cern-ohl). This documentation is distributed WITHOUT ANY
// EXPRESS OR IMPLIED WARRANTY, INCLUDING OF MERCHANTABILITY, SATISFACTORY QUALITY AND FITNESS FOR A
// PARTICULAR PURPOSE. Please see the CERN-OHL-P v2 for applicable conditions.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
`timescale 1ps / 1ps

module sim;

	// Parameters
	parameter CLK_CYCLE = 10000;	// 100MHz
	parameter CT_WIDTH = 8;
	parameter CONTINUE1 = 1;
//	parameter CONTINUE1 = 0;
	parameter CONTINUE2 = 0;
	parameter CARRY_START1 = 0;
	parameter CARRY_START2 = 1;
	
	// Inputs
	reg CLOCK = 1'b0;
	reg RESET = 1'b0;
	reg [CT_WIDTH-1:0] IN_COUNT1 = 0;
	reg [CT_WIDTH-1:0] IN_COUNT2 = 5;
	reg IN_COUNT_CLR = 1'b0;
	reg IN_COUNT_RUN = 1'b0;
	reg IN_COUNT_STB = 1'b0;
	reg [CT_WIDTH-1:0] IN_COMP_LOW1 = 1;
	reg [CT_WIDTH-1:0] IN_COMP_HIGH1 = 9;
	reg [CT_WIDTH-1:0] IN_COMP_LOW2 = 3;
	reg [CT_WIDTH-1:0] IN_COMP_HIGH2 = 4;

	// Outputs
	wire [CT_WIDTH-1:0] OUT_COUNT1;
	wire [CT_WIDTH-1:0] OUT_COUNT2;
	wire OUT_CARRY1;
	wire OUT_CARRY2;
	wire OUT_COMP1;
	wire OUT_COMP2;
	wire OUT_COUNT_STB1;
	wire OUT_COUNT_STB2;

	// Instantiate the Unit Under Test (UUT)
	counter # (
		.CLOCK_POL(1),
		.RESET_POL(0),
		.ARESET(0),
		.CT_WIDTH(CT_WIDTH),
		.CONTINUE(CONTINUE1),
		.CARRY_START(CARRY_START1)
	) dut1 (
		.CLOCK(CLOCK), 
		.RESET(RESET), 
		.IN_COUNT(IN_COUNT1),
		.IN_COUNT_CLR(IN_COUNT_CLR),
		.IN_COUNT_RUN(IN_COUNT_RUN),
		.IN_COUNT_STB(IN_COUNT_STB),
		.IN_COMP_LOW(IN_COMP_LOW1), 
		.IN_COMP_HIGH(IN_COMP_HIGH1), 
		.OUT_COUNT(OUT_COUNT1), 
		.OUT_CARRY(OUT_CARRY1),
		.OUT_COMP(OUT_COMP1),
		.OUT_COUNT_STB(OUT_COUNT_STB1)
	);

	counter # (
		.CLOCK_POL(1),
		.RESET_POL(0),
		.ARESET(1),
		.CT_WIDTH(CT_WIDTH),
		.CONTINUE(CONTINUE2),
		.CARRY_START(CARRY_START2)
	) dut2 (
		.CLOCK(CLOCK), 
		.RESET(RESET), 
		.IN_COUNT(IN_COUNT2), 
		.IN_COUNT_CLR(IN_COUNT_CLR),
		.IN_COUNT_RUN(IN_COUNT_RUN),
		.IN_COUNT_STB(OUT_CARRY1), 
		.IN_COMP_LOW(IN_COMP_LOW2), 
		.IN_COMP_HIGH(IN_COMP_HIGH2), 
		.OUT_COUNT(OUT_COUNT2), 
		.OUT_CARRY(OUT_CARRY2),
		.OUT_COMP(OUT_COMP2),
		.OUT_COUNT_STB(OUT_COUNT_STB2)
	);

	task task_delay (
			input integer value
		);
		begin
			repeat (value)
				@(posedge CLOCK);
			#1;
		end
	endtask

	initial begin
		$dumpfile("outflow/counter.vcd");
		$dumpvars(0, sim);
	end

	always #(CLK_CYCLE / 2) CLOCK <= ~CLOCK;

	initial begin
		#(CLK_CYCLE * 5.25);
		RESET = 1'b1;		

		task_delay(10);
		
		IN_COUNT_STB = 'b1;			task_delay(10);
		IN_COUNT_RUN = 'b1;			task_delay(20);
		IN_COUNT1 = 1;				task_delay(30);
		IN_COUNT1 = 10;				task_delay(50);
		IN_COUNT_STB = 'b0;			task_delay(10);
		IN_COUNT_STB = 'b1;			task_delay(45);
		IN_COUNT2 = 10;				task_delay(30);
		IN_COUNT2 = 8;				task_delay(20);
		IN_COUNT1 = 5;				task_delay(21);
		IN_COUNT2 = 9;				task_delay(20);
		IN_COUNT2 = 8;				task_delay(10);
		IN_COUNT_RUN = 'b0;			task_delay(10);
		IN_COUNT_CLR = 'b1;			task_delay(10);
		IN_COUNT_CLR = 'b0;			task_delay(100);

		$finish;
	end

endmodule
