//% @file d_ff.v
//% @brief D flip-flop module
//% @date 2023/3/31
//% @author Hiroyuki Uesugi
//% @copyright Since 2011 RIGHTxLIGHT Co., Ltd.

module d_ff # (
		parameter CLOCK_POL = 1,
		parameter RESET_POL = 0,
		parameter ARESET = 1,
		parameter DATA_WIDTH = 1
	) (
		input CLOCK,
		input RESET,
		input [DATA_WIDTH-1:0] IN_DATA,
		output [DATA_WIDTH-1:0] OUT_DATA
	);

	reg [DATA_WIDTH-1:0] data_reg;

	wire clk, rst, a_rst, s_rst;

	assign clk = (CLOCK_POL == 0)? ~CLOCK : CLOCK;
	assign rst = (RESET_POL == 0)? ~RESET : RESET;
	assign a_rst = (ARESET != 0)? rst : 1'b0;
	assign s_rst = (ARESET == 0)? rst : 1'b0;
	
	assign OUT_DATA = data_reg;

	always @(posedge clk or posedge a_rst) begin
		if (a_rst)
			data_reg <= {DATA_WIDTH{1'b0}};
		else if (s_rst)
			data_reg <= {DATA_WIDTH{1'b0}};
		else
			data_reg <= IN_DATA;
	end

endmodule
