// The CERN-OHL-P v2 License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// This source describes Open Hardware and is licensed under the CERN-OHL-P v2.
// You may redistribute and modify this documentation and make products using it under the terms of
// the CERN-OHL-P v2 (https:/cern.ch/cern-ohl). This documentation is distributed WITHOUT ANY
// EXPRESS OR IMPLIED WARRANTY, INCLUDING OF MERCHANTABILITY, SATISFACTORY QUALITY AND FITNESS FOR A
// PARTICULAR PURPOSE. Please see the CERN-OHL-P v2 for applicable conditions.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
`timescale 1ps / 1ps

module sim;

	// Parameters
	parameter CLK_CYCLE = 10000;	// 100MHz
	parameter DATA_WIDTH = 8;

	// Inputs
	reg CLOCK = 1'b0;
	reg RESET_N = 1'b0;
	reg IN_DATA = 1'b0;

	// Outputs
	wire OUT_RISE;
	wire OUT_FALL;

	edge_det #(
		.CLOCK_POL(1),
		.RESET_POL(0),
		.ARESET(1)
	) dut1 (
		.CLOCK(CLOCK), 
		.RESET(RESET_N), 
		.IN_DATA(IN_DATA),
		.OUT_RISE(OUT_RISE),
		.OUT_FALL(OUT_FALL)
	);

	initial begin
		$dumpfile("outflow/edge_det.vcd");
		$dumpvars(0, sim);
	end

	always #(CLK_CYCLE / 2) CLOCK <= ~CLOCK;

	task task_delay (
			input integer value
		);
		begin
			repeat (value)
				@(posedge CLOCK);
			#1;
		end
	endtask

	task inputData (
			input data
		);
		begin
			@(posedge CLOCK)
			#1	IN_DATA <= data;
		end
	endtask

	initial begin

		#(CLK_CYCLE * 5.25);
		RESET_N = 1'b1;		
		#(CLK_CYCLE * 5);

		inputData(1);
		task_delay(10);

		inputData(0);
		task_delay(10);

		inputData(1);
		inputData(0);
		inputData(1);
		task_delay(10);

		inputData(0);
		task_delay(10);

		$finish;
	end

endmodule
