// The MIT License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
#include <stdbool.h>

#include <bsp.h>
#include <plic.h>
#include <clint.h>
#include <riscv.h>

#include "timer.h"
#include "power.h"
#include "uart.h"
#include "test.h"
#include "gpio_test.h"

extern void trap_entry(void);

static void crash(void)
{
    bsp_printf("\n*** CRASH ***\n");
    while(1);
}

static void machineExternalinterrupt(void)
{
    uint32_t claim;

    while (claim = plic_claim(BSP_PLIC, BSP_PLIC_CPU_0)) {
        switch (claim){
        case SYSTEM_PLIC_SYSTEM_UART_0_IO_INTERRUPT:
        	uartInterrupt();
        	break;
		case SYSTEM_PLIC_SYSTEM_GPIO_0_IO_INTERRUPTS_0:
			gpioTestInterrupt(BIT_0);
			break;
		case SYSTEM_PLIC_SYSTEM_GPIO_0_IO_INTERRUPTS_1:
			gpioTestInterrupt(BIT_1);
			break;
		case SYSTEM_PLIC_SYSTEM_GPIO_1_IO_INTERRUPTS_0:
			gpioTestInterrupt(BIT_16);
			break;
		case SYSTEM_PLIC_SYSTEM_GPIO_1_IO_INTERRUPTS_1:
			gpioTestInterrupt(BIT_17);
			break;
		case SYSTEM_PLIC_USER_INTERRUPT_A_INTERRUPT:
			powerInterruptProc();
			break;
        default:
        	crash();
        	break;
        }
        plic_release(BSP_PLIC, BSP_PLIC_CPU_0, claim);
    }
}

void trap(void)
{
    int32_t mcause = csr_read(mcause);
    int32_t interrupt = mcause < 0;
    int32_t cause     = mcause & 0xF;

    if (interrupt) {
        switch (cause) {
        case CAUSE_MACHINE_EXTERNAL:
        	machineExternalinterrupt();
        	break;
        case CAUSE_MACHINE_TIMER:
        	machineTimerInterrupt();
        	break;
        default:
        	crash();
        	break;
        }
    } else {
        crash();
    }
}

void interruptInit(void)
{
    plic_set_threshold(BSP_PLIC, BSP_PLIC_CPU_0, 0);

	csr_write(mtvec, trap_entry);
	csr_set(mie, MIE_MEIE | MIE_MTIE);	// Enable machine external interrupt and machine timer interrupt
	csr_write(mstatus, csr_read(mstatus) | MSTATUS_MPP | MSTATUS_MIE);
}



