// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// The MIT License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
#include <stdbool.h>

#include <bsp.h>
#include <plic.h>
#include <clint.h>
#include <riscv.h>
#include <gpio.h>

#include "test.h"
#include "lib/apb3.h"
#include "lib/gpio.h"

static u32 gpioInterruptBit;

typedef struct {
	int gpi;
	int gpo;
} loopback_t;

#define	RASPBERRY	1
#define	ARDUINO		2

#define CONNECTOR_TYPE		RASPBERRY
//#define CONNECTOR_TYPE	ARDUINO

#if CONNECTOR_TYPE == RASPBERRY
const loopback_t LOOPBACK_TBL[] = {
	{	0,		14	},		// gpio0 bit0 irq
	{	1,		15	},		// gpio0 bit1 irq
	{	16,		2	},		// gpio1 bit0 irq
	{	17,		3	},		// gpio1 bit1 irq
	{	4,		18	},
	{	5,		19	},
	{	20,		6	},
	{	21,		7	},
	{	8,		22	},
	{	9,		23	},
	{	24,		10	},
	{	25,		11	},
	{	12,		26	},
	{	13,		27	}
};
#define TEST_OUTPUT_BIT	(BIT_14 | BIT_15 | BIT_2 | BIT_3 | BIT_18 | BIT_19 | BIT_6 | BIT_7 | BIT_22 | BIT_23 | BIT_10 | BIT_11 | BIT_26 | BIT_27)

#elif CONNECTOR_TYPE == ARDUINO
const loopback_t LOOPBACK_TBL[] = {
	{	0,		4	},		// gpio0 bit0 irq
	{	1,		5	},		// gpio0 bit1 irq
	{	2,		6	},
	{	3,		7	},
	{	8,		12	},
	{	9,		13	},
	{	10,		20	},
	{	11,		21	},
	{	15,		18	},
	{	16,		19	},		// gpio1 bit0 irq
	{	17,		14	}		// gpio1 bit1 irq
};
#define TEST_OUTPUT_BIT	(BIT_4 | BIT_5 | BIT_6 | BIT_7 | BIT_12 | BIT_13 | BIT_14 | BIT_18 | BIT_19 | BIT_20 | BIT_21)

#endif

void gpioTestInterrupt(u32 bit)
{
	gpioInterruptBit |= bit;
	bsp_printf("GPIO interrupt bit : 0x%08X\n", bit);
}

bool gpioTest(test_state_t test_state)
{
	static bool gpioError = false;

	if (test_state == test_state_end) {
		SLED_BLK(1, 1);
		gpioError = false;
		return false;
	}

	if (test_state == test_state_start) {
		SLED_YEL(1, 1);

		gpio32_clearOutput(0xFFFFFFFF);
		gpio32_setOutputEnable(TEST_OUTPUT_BIT);
		gpioInterruptBit = 0;
		for (int i = 0; i < sizeof(LOOPBACK_TBL) / sizeof(loopback_t); i = i + 1) {
			gpio32_setOutput(1<<(LOOPBACK_TBL[i].gpo));
			bsp_uDelay(100000);
			if (gpio32_getInput(0x0FFFFFFF) == (1<<(LOOPBACK_TBL[i].gpi) | 1<<(LOOPBACK_TBL[i].gpo))) {
				bsp_printf("test[%d] passed!(%d -> %d)\n", i, LOOPBACK_TBL[i].gpo, LOOPBACK_TBL[i].gpi);
			} else {
				bsp_printf("test[%d] failed!!!(%d -> %d)\n", i, LOOPBACK_TBL[i].gpo, LOOPBACK_TBL[i].gpi);
				SLED_RED(1, 1);
				gpioError = true;
			}
			gpio32_clearOutput(0xFFFFFFFF);
			bsp_uDelay(100000);
		}
		if (gpioInterruptBit != (BIT_0 | BIT_1 | BIT_16 | BIT_17)) {
			bsp_printf("interrupt test failed!!!(%x)\n", gpioInterruptBit);
			gpioError = true;
			SLED_RED(1, 1);
		}
		if (!gpioError) {
			SLED_GRN(1, 1);
		}
		gpio32_clearOutputEnable(TEST_OUTPUT_BIT);
	}

	return true;
}

void gpioTestInit(void)
{
	plic_set_enable(BSP_PLIC, BSP_PLIC_CPU_0, SYSTEM_PLIC_SYSTEM_GPIO_0_IO_INTERRUPTS_0, 1);
	plic_set_enable(BSP_PLIC, BSP_PLIC_CPU_0, SYSTEM_PLIC_SYSTEM_GPIO_0_IO_INTERRUPTS_1, 1);
	plic_set_enable(BSP_PLIC, BSP_PLIC_CPU_0, SYSTEM_PLIC_SYSTEM_GPIO_1_IO_INTERRUPTS_0, 1);
	plic_set_enable(BSP_PLIC, BSP_PLIC_CPU_0, SYSTEM_PLIC_SYSTEM_GPIO_1_IO_INTERRUPTS_1, 1);
	plic_set_priority(BSP_PLIC, SYSTEM_PLIC_SYSTEM_GPIO_0_IO_INTERRUPTS_0, 1);
	plic_set_priority(BSP_PLIC, SYSTEM_PLIC_SYSTEM_GPIO_0_IO_INTERRUPTS_1, 1);
	plic_set_priority(BSP_PLIC, SYSTEM_PLIC_SYSTEM_GPIO_1_IO_INTERRUPTS_0, 1);
	plic_set_priority(BSP_PLIC, SYSTEM_PLIC_SYSTEM_GPIO_1_IO_INTERRUPTS_1, 1);
	gpio_setInterruptRiseEnable(GPIO0, BIT_1 | BIT_0);
	gpio_setInterruptRiseEnable(GPIO1, BIT_1 | BIT_0);
}

