// The MIT License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
#pragma once

#define FACTORY				true		// Execute only during factory inspection.
//#define FACTORY					false		// Execute only during factory inspection.

#define DUMMY_READ				true		// Improves the measurement accuracy of AD converters.

#define AD_CH0					0
#define AD_CH1					1
#define AD_CH2					2
#define AD_CH3					3
#define AD_CH4					4
#define AD_CH5					5
#define AD_VIN_CURR				6
#define AD_VBUS_CURR			7

#define AD_CHANNELS				8

#define DA_CHANNELS				1			// MCP48FEB11
//#define DA_CHANNELS			2			// MCP48FEB12

typedef union {
	struct {
		u8	b7_0				:8;
		u8	b11_8				:4;
		u8	sel					:3;
		u8	single				:1;
		u8	start				:1;
		u8	b23_17				:7;
	};
	struct {
		u16	data				:10;
		u16	b15_10				:6;
		u8	b23_16				:8;
	};
	u8	arr[3];
} mcp3008_t;

extern uint32_t mcp3008_read(mcp3008_t s);
#define MCP3008_READ(ch)	mcp3008_read((mcp3008_t){.b23_17 = 0, .start = 1, .single = 1, .sel = ch, .b11_8 = 0, .b7_0 = 0})


typedef struct {
	u16 d		:10;
	u16 b15_10	:6;
} mcp48_dac_t;

// mcp48_vref_t.vr0
#define VDD_UNBUFFERED			0x0
#define VREF_BAND_GAP			0x1			// HVC is not supported.
#define VREF_UNBUFFERED			0x2
#define VREF_BUFFERED			0x3

typedef struct {
	u8 vr0		:2;
	u8 b7_2		:6;
	u8 b15_8	:8;
} mcp48_vref_t;

// mcp48_power_down_t.pd0
#define NORMAL_OPERATION		0x0
#define POWER_DOWN_VOUT_1KOHM	0x1
#define POWER_DOWN_VOUT_100KOHM	0x2
#define POWER_DOWN_VOUT_OPEN	0x3

typedef struct {
	u8 pd0		:2;
	u8 b7_2		:6;
	u8 b15_8	:8;
} mcp48_power_down_t;

typedef struct {
	u8 b5_0		:6;
	u8 eewa		:1;		// Volatile memory only
	u8 por		:1;		// Volatile memory only
	u8 g0		:1;
	u8 b15_9	:7;
} mcp48_gain_stat_t;

typedef struct {
	u8 b7_0		:8;
	u8 g0		:1;
	u8 b15_9	:7;
} mcp48_gain_t;

// mcp48_wiperlock_t.wl0
#define WIPER_CONFIG_LOCK				0x3
#define VOLATILE_CONFIG_UNLOCK			0x2
#define VOLATILE_WIPER_CONFIG_UNLOCK	0x1
#define WIPER_CONFIG_UNLOCK				0x0

typedef struct {
	u8 wl0		:2;
	u8 b7_2		:6;
	u8 b15_8	:8;
} mcp48_wiperlock_t;

// mcp48feb_t.ad
#define MCP48_DAC0_ADDR			0x00
#if DA_CHANNELS == 2
#define MCP48_DAC1_ADDR			0x01	// DAC1 is not supported.
#endif
#define MCP48_VREF_ADDR			0x08
#define MCP48_POW_DOWN_ADDR		0x09
#define MCP48_GAIN_STAT_ADDR	0x0A
#define MCP48_WIPERLOCK_ADDR	0x0B
#define MCP48_NV_DAC0_ADDR		0x10
#if DA_CHANNELS == 2
#define MCP48_NV_DAC1_ADDR	0x11		// DAC1 is not supported.
#endif
#define MCP48_NV_VREF_ADDR		0x18
#define MCP48_NV_POW_DOWN_ADDR	0x19
#define MCP48_NV_GAIN_ADDR		0x1A

// mcp48feb_t.c
#define MCP48_CMD_WRITE			0x0
//#define MCP48_CMD_ENABLE		0x1		// HVC is not supported.
//#define MCP48_CMD_DISABLE		0x2		// HVC is not supported.
#define MCP48_CMD_READ			0x3

#define MCP48_CMD_VALID			0x1
#define MCP48_CMD_INVALID		0x0

typedef union {
	struct {
		union {
			mcp48_dac_t dac;
			mcp48_vref_t vr;
			mcp48_power_down_t pd;
			mcp48_gain_stat_t gs;
			mcp48_gain_t g;
			mcp48_wiperlock_t wl;
			u16 d;
		};
		u8 cmderr	:1;
		u8 c		:2;
		u8 ad		:5;
	};
	u8	arr[3];
} mcp48feb_t;

extern mcp48feb_t mcp48_write_read(mcp48feb_t s);
extern mcp48feb_t mcp48_nvm_write_read(mcp48feb_t s);
#if FACTORY // Execute only during factory inspection.
extern void mcp48feb_factory_set(void);
#endif

#define MCP48FEB_SET_DAC1(v)	mcp48_write_read((mcp48feb_t){.ad = MCP48_DAC0_ADDR, .c = MCP48_CMD_WRITE, .cmderr = MCP48_CMD_VALID, .dac.b15_10 = 0, .dac.d = v})
#if DA_CHANNELS == 2
#define MCP48FEB_SET_DAC2(v)	mcp48_write_read((mcp48feb_t){.ad = MCP48_DAC1_ADDR, .c = MCP48_CMD_WRITE, .cmderr = MCP48_CMD_VALID, .dac.b15_10 = 0, .dac.d = v})
#endif
#define MCP48FEB_SET_VREF(v)	mcp48_write_read((mcp48feb_t){.ad = MCP48_VREF_ADDR, .c = MCP48_CMD_WRITE, .cmderr = MCP48_CMD_VALID, .vr.b15_8 = 0, .vr.b7_2 = 0, .vr.vr0 = v})
#define MCP48FEB_SET_PD(v)		mcp48_write_read((mcp48feb_t){.ad = MCP48_POW_DOWN_ADDR, .c = MCP48_CMD_WRITE, .cmderr = MCP48_CMD_VALID, .pd.b15_8 = 0, .pd.b7_2 = 0, .pd.pd0 = v})
#define MCP48FEB_GET_STAT()		mcp48_write_read((mcp48feb_t){.ad = MCP48_GAIN_STAT_ADDR, .c = MCP48_CMD_READ, .cmderr = MCP48_CMD_VALID, .d = 0})

extern void adda_ext_vref_sel(bool ext);

#define PRODUCT_OVER_SUM(r1, r2)	((r1 * r2) / (r1 + r2))
#define RCLIM_500MA		1000.0f
#define RCLIM_1500MA	PRODUCT_OVER_SUM(RCLIM_500MA, 499.0f)
#define RCLIM_2000MA	PRODUCT_OVER_SUM(RCLIM_500MA, 332.0f)
#define RCLIM_3000MA	PRODUCT_OVER_SUM(RCLIM_1500MA, 332.0f)
#define RCLIM_VIN		169.0f

#define NON_INV_AMP_GAIN(r1, r2)	(1 + r2 / r1)
#define CMON_GAIN		NON_INV_AMP_GAIN(15.0e3, 75.0e3)

extern uint32_t calc_mon_current_ma(uint32_t ad_val, uint32_t adc_res);
