// The MIT License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
#pragma once

#define APB0		IO_APB_SLAVE_0_INPUT

#define VERSION_REG_ADDR			0x0
#define DATE_REG_ADDR				0x4

#define CONTROL_REG_ADDR			0x10
#define STATUS_REG_ADDR				CONTROL_REG_ADDR
#define STATUS_CHG_REG_ADDR			0x14
#define STATUS_CLR_CHG_REG_ADDR		STATUS_CHG_REG_ADDR
#define CONTROL_SET_REG_ADDR		0x18
#define CONTROL_CLR_REG_ADDR		0x1C

#define INT_UPEG_REG_ADDR			0x20
#define INT_DNEG_REG_ADDR			0x24
#define INT_REG_ADDR				0x28
#define INT_CLR_REG_ADDR			INT_REG_ADDR

#define SLED_REG_ADDR				0x30

readReg_u32 (apb3_readVersion            , VERSION_REG_ADDR)
readReg_u32 (apb3_readDate               , DATE_REG_ADDR)
writeReg_u32(apb3_writeControl           , CONTROL_REG_ADDR)
readReg_u32 (apb3_readStatus             , STATUS_REG_ADDR)
readReg_u32 (apb3_readStatusChange       , STATUS_CHG_REG_ADDR)
writeReg_u32(apb3_clearStatusChange      , STATUS_CLR_CHG_REG_ADDR)
writeReg_u32(apb3_setControl             , CONTROL_SET_REG_ADDR)
writeReg_u32(apb3_clearControl           , CONTROL_CLR_REG_ADDR)

writeReg_u32(apb3_setInterruptUpedge     , INT_UPEG_REG_ADDR)
writeReg_u32(apb3_setInterruptDownedge   , INT_DNEG_REG_ADDR)
readReg_u32 (apb3_readInterrupt          , INT_REG_ADDR)
writeReg_u32(apb3_clearInterrupt         , INT_CLR_REG_ADDR)

writeReg_u32(apb3_setSled		         , SLED_REG_ADDR)
readReg_u32 (apb3_readSled		         , SLED_REG_ADDR)

#define STAT_PWR_SW_N_BIT		BIT_0
#define STAT_VIN_MON_N			BIT_1
#define STAT_VBUS_MON_N			BIT_2
#define CONT_POW_CNT_BIT		BIT_3
#define CONT_USB_PWR_DIS		BIT_4
#define CONT_ISET_1R0A_N		BIT_5
#define CONT_ISET_1R5A_N		BIT_6
#define STAT_TYPEC_SWMONI		BIT_7
#define STAT_TYPEC_TCC0			BIT_8
#define STAT_TYPEC_TCC1			BIT_9
#define CONT_USB_VCP_RTS		BIT_10
#define STAT_USB_VCP_CTS		BIT_11
#define STAT_USB_CHR0			BIT_12
#define STAT_USB_CHR1			BIT_13
#define STAT_USB_CHR_EN			BIT_14
#define STAT_USB_SUSPENDED		BIT_15
#define STAT_SD_DET_N			BIT_16
#define CONT_AREF_EXT_SEL_N		BIT_17
#define CONT_PWR_LED_R_N		BIT_18
#define CONT_PWR_LED_G_N		BIT_19
#define CONT_PWR_LED_B_N		BIT_20
#define CONT_LVDS_TXPLL_RST_N	BIT_21
#define STAT_LVDS_TXPLL_LOCKED	BIT_22
#define CONT_LVDS_RXPLL_RST_N	BIT_23
#define STAT_LVDS_RXPLL_LOCKED	BIT_24
#define CONT_LVDS_TEST1			BIT_25
#define CONT_LVDS_TEST2			BIT_26
#define CONT_LVDS_TEST3			BIT_27
#define CONT_LVDS_TEST4			BIT_28
#define CONT_LVDS_TEST5			BIT_29
#define STAT_LVDS_TEST_RX		BIT_30
#define STAT_LVDS_TEST_ERR		BIT_31

typedef union {
	struct {
		char rev;
		char minor[2];
		char major;
	};
} fpga_version_t;

typedef union {
	struct {
		u8 dd;
		u8 mm;
		u16 yyyy;
	};
} fpga_date_t;

typedef union {
	u32 u32;
	struct {
		u32						:3;		// bit0-2
		u32 pwr_cnt				:1;		// bit3
		u32 usb_pwr_dis			:1;		// bit4
		u32 iset_1r0a_n			:1;		// bit5
		u32 iset_1r5a_n			:1;		// bit6
		u32						:3;		// bit7-9
		u32 usb_vcp_rts			:1;		// bit10
		u32						:6;		// bit11-16
		u32 aref_ext_sel_n		:1;		// bit17
		u32 pwr_led_r_n			:1;		// bit18
		u32 pwr_led_g_n			:1;		// bit19
		u32 pwr_led_b_n			:1;		// bit20
		u32 lvds_txpll_rst_n	:1;		// bit21
		u32						:1;		// bit22
		u32 lvds_rxpll_rst_n	:1;		// bit23
		u32						:1;		// bit24
		u32 lvds_test1			:1;		// bit25
		u32 lvds_test2			:1;		// bit26
		u32 lvds_test3			:1;		// bit27
		u32 lvds_test4			:1;		// bit28
		u32 lvds_test5			:1;		// bit29
		u32						:2;		// bit30-31
	};
	struct {
		u32						:25;	// bit0-24
		u32 lvds_test			:5;		// bit25-29
		u32						:2;		// bit30-31
	};
} control_t;

typedef union {
	u32 u32;
	struct {
		u32 pwr_sw_n			:1;		// bit0
		u32 vin_mon_n			:1;		// bit1
		u32 vbus_mon_n			:1;		// bit2
		u32						:4;		// bit3-6
		u32 typec_swmoni		:1;		// bit7
		u32 typec_tcc0			:1;		// bit8
		u32 typec_tcc1			:1;		// bit9
		u32						:1;		// bit10
		u32 usb_vcp_cts			:1;		// bit11
		u32 usb_chr0			:1;		// bit12
		u32 usb_chr1			:1;		// bit13
		u32 usb_chr_en			:1;		// bit14
		u32 usb_suspended		:1;		// bit15
		u32 sd_det_n			:1;		// bit16
		u32						:5;		// bit17-21
		u32 lvds_txpll_locked	:1;		// bit22
		u32						:1;		// bit23
		u32 lvds_rxpll_locked	:1;		// bit24
		u32						:5;		// bit25-29
		u32 lvds_test_rx		:1;		// bit30
		u32 lvds_test_err		:1;		// bit31
	};
	struct {
		u32						:8;		// bit0-7
		u32 typec_tcc			:2;		// bit8-9
		u32						:2;		// bit10-11
		u32 usb_chr				:2;		// bit12-13
		u32 					:18;	// bit14-31
	};
} status_t;

typedef union {
	u32 u32;
	struct {
		u32	b					:8;		// bit16:23
		u32 r					:8;		// bit8-15
		u32	g					:8;		// bit0-7
		u32 ad					:2;		// bit24-25
		u32						:5;		// bit26-30
		u32 set					:1;		// bit31
	};
	struct {
		u32						:31;
		u32 busy				:1;		// bit31
	};
} sled_t;

#define SLED_RED(a,s)		set_led((sled_t){.ad=a, .r=0x80, .g=0x00, .b=0x00, .set=s})
#define SLED_GRN(a,s)		set_led((sled_t){.ad=a, .r=0x00, .g=0x80, .b=0x00, .set=s})
#define SLED_BLU(a,s)		set_led((sled_t){.ad=a, .r=0x00, .g=0x00, .b=0x80, .set=s})
#define SLED_CYN(a,s)		set_led((sled_t){.ad=a, .r=0x00, .g=0x40, .b=0x40, .set=s})
#define SLED_MAG(a,s)		set_led((sled_t){.ad=a, .r=0x40, .g=0x00, .b=0x40, .set=s})
#define SLED_YEL(a,s)		set_led((sled_t){.ad=a, .r=0x40, .g=0x40, .b=0x00, .set=s})
#define SLED_WHT(a,s)		set_led((sled_t){.ad=a, .r=0x20, .g=0x20, .b=0x20, .set=s})
#define SLED_BLK(a,s)		set_led((sled_t){.ad=a, .r=0x00, .g=0x00, .b=0x00, .set=s})

extern void set_led(sled_t led);
extern void apb3_dispVerDate(void);
