// The MIT License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
#pragma once

#define FL_SPI SYSTEM_SPI_1_IO_CTRL

//　The supported flash memory is Winbond's W25Q32JVPI.

#define FAST_READ							false

#define FLASH_POWER_DOWN_TO_STAND_BY_US		5
#define FLASH_WRITE_TIMEOUT_MS				3
#define FLASH_SECTOR_ERASE_TIMEOUT_MS		400
#define FLASH_BLOCK_ERASE_TIMEOUT_MS		3200

#define FLASH_PAGE_SIZE						256
#define FLASH_PAGE_NUMS						16
#define FLASH_SECT_NUMS						16
#define FLASH_BLOCK_NUMS					64

typedef union {
	u8				u8[FLASH_PAGE_SIZE];
	u16				u16[FLASH_PAGE_SIZE/sizeof(u16)];
	u32				u32[FLASH_PAGE_SIZE/sizeof(u32)];
} FLASH_PAGE_t;

typedef union {
	FLASH_PAGE_t	page[FLASH_PAGE_NUMS];
	u8				u8[sizeof(FLASH_PAGE_t)*FLASH_PAGE_NUMS];
	u16				u16[sizeof(FLASH_PAGE_t)*FLASH_PAGE_NUMS/sizeof(u16)];
	u32				u32[sizeof(FLASH_PAGE_t)*FLASH_PAGE_NUMS/sizeof(u32)];
} FLASH_SECTOR_t;

typedef union {
	FLASH_SECTOR_t	sector[FLASH_SECT_NUMS];
	u8				u8[sizeof(FLASH_SECTOR_t)*FLASH_SECT_NUMS];
	u16				u16[sizeof(FLASH_SECTOR_t)*FLASH_SECT_NUMS/sizeof(u16)];
	u32				u32[sizeof(FLASH_SECTOR_t)*FLASH_SECT_NUMS/sizeof(u32)];
} FLASH_BLOCK_t;

typedef union {
	FLASH_BLOCK_t	block[FLASH_BLOCK_NUMS];
	u8				u8[sizeof(FLASH_BLOCK_t)*FLASH_BLOCK_NUMS];
	u16				u16[sizeof(FLASH_BLOCK_t)*FLASH_BLOCK_NUMS/sizeof(u16)];
	u32				u32[sizeof(FLASH_BLOCK_t)*FLASH_BLOCK_NUMS/sizeof(u32)];
} FLASH_MEMORY_t;

typedef union {
	u8	u8;
	struct {
		u8 busy		:1;
		u8 wel		:1;
		u8 bp0		:1;
		u8 bp1		:1;
		u8 bp2		:1;
		u8 tb		:1;
		u8 sec		:1;
		u8 srp		:1;
	};
} FLASH_ST1_t;

typedef union {
	void*	p;
	u32		u32;
	u8		u8[4];
} FLASH_ADDR_t;

typedef struct {
	FLASH_ADDR_t	addr;
	u8*				data;
	size_t			len;
} FLASH_t;

enum {
	FLASH_WRITE_ENABLE = 0x06,
	FLASH_RELEASE_POWER_DOWN = 0xAB,
	FLASH_READ_DATA = 0x03,
	FLASH_FAST_READ = 0x0B,
	FLASH_PAGE_PROGRAM = 0x02,
	FLASH_SECTOR_ERASE = 0x20,
	FLASH_BLOCK_ERASE = 0xD8,
	FLASH_READ_STATUS_REG1 = 0x05,
	FLASH_POWER_DOWN = 0xB9
} FLASH_INS_t;

#define FLASH_ADDR(b,s,p)		(&((FLASH_MEMORY_t*)NULL)->block[b].sector[s].page[p])

extern void FlashWriteEnable(void);
extern void FlashReleasePowerDown(void);
extern void FlashReadData(FLASH_t arg);
extern bool FlashPageProgram(FLASH_t arg);
extern bool FlashSectorErase(FLASH_ADDR_t Addr);
extern bool FlashBlockErase(FLASH_ADDR_t Addr);
extern FLASH_ST1_t FlashReadStatusReg1();
extern void FlashPowerDown(void);
