// The MIT License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
#include <stdbool.h>

#include <bsp.h>
#include <gpio.h>

#include "gpio.h"
#include "../interrupt.h"

void gpio32_setOutputEnable(u32 bit)
{
	MACHINE_INTERRUPT_STORE_AND_DISABLE();
	if (bit & 0x0000FFFF) 	gpio_setOutputEnable(GPIO0, gpio_getOutputEnable(GPIO0) | (bit & 0x0000FFFF));
	if (bit & 0xFFFF0000) 	gpio_setOutputEnable(GPIO1, gpio_getOutputEnable(GPIO1) | ((bit >> 16) & 0x0000FFFF));
	MACHINE_INTERRUPT_RESTORE();
}

void gpio32_clearOutputEnable(u32 bit)
{
	MACHINE_INTERRUPT_STORE_AND_DISABLE();
	if (bit & 0x0000FFFF) 	gpio_setOutputEnable(GPIO0, gpio_getOutputEnable(GPIO0) & (~(bit & 0x0000FFFF)));
	if (bit & 0xFFFF0000) 	gpio_setOutputEnable(GPIO1, gpio_getOutputEnable(GPIO1) & (~((bit >> 16) & 0x0000FFFF)));
	MACHINE_INTERRUPT_RESTORE();
}

void gpio32_setOutput(u32 bit)
{
	MACHINE_INTERRUPT_STORE_AND_DISABLE();
	if (bit & 0x0000FFFF) 	gpio_setOutput(GPIO0, gpio_getOutput(GPIO0) | (bit & 0x0000FFFF));
	if (bit & 0xFFFF0000) 	gpio_setOutput(GPIO1, gpio_getOutput(GPIO1) | ((bit >> 16) & 0x0000FFFF));
	MACHINE_INTERRUPT_RESTORE();
}

void gpio32_clearOutput(u32 bit)
{
	MACHINE_INTERRUPT_STORE_AND_DISABLE();
	if (bit & 0x0000FFFF) 	gpio_setOutput(GPIO0, gpio_getOutput(GPIO0) & (~(bit & 0x0000FFFF)));
	if (bit & 0xFFFF0000) 	gpio_setOutput(GPIO1, gpio_getOutput(GPIO1) & (~((bit >> 16) & 0x0000FFFF)));
	MACHINE_INTERRUPT_RESTORE();
}

u32 gpio32_getOutput(u32 bit)
{
	u32 retval;
	MACHINE_INTERRUPT_STORE_AND_DISABLE();
	retval = (((gpio_getOutput(GPIO0) & 0x0000FFFF) | ((gpio_getOutput(GPIO1) << 16) & 0xFFFF0000)) & bit);
	MACHINE_INTERRUPT_RESTORE();
	return retval;
}

u32 gpio32_getInput(u32 bit)
{
	u32 retval;
	MACHINE_INTERRUPT_STORE_AND_DISABLE();
	retval = (((gpio_getInput(GPIO0) & 0x0000FFFF) | ((gpio_getInput(GPIO1) << 16) & 0xFFFF0000)) & bit);
	MACHINE_INTERRUPT_RESTORE();
	return retval;
}

void gpio32_init(void)
{
	gpio32_clearOutputEnable(0xFFFFFFFF);
	gpio32_setOutputEnable(LED1_BIT | LED2_BIT | LED3_BIT | LED4_BIT);
	gpio32_clearOutput(0xFFFFFFFF);
}
