// The MIT License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
#pragma once

#include <i2c.h>

    /*
     *  I2C Write data without register address
     */
    static void i2c_writeData(u32 reg, u8 slaveAddr, u8 *data, u32 length){
        i2c_masterStartBlocking(reg);               // Send start sequence
        i2c_txByte(reg, slaveAddr | I2C_WRITE);     // write device address byte with write bit
        i2c_txNackBlocking(reg);                    // send nack bit
        if(length > 1){
            for(int i = 0; i < length - 1; i++){
                    i2c_txByte(reg, data[i]);       // send 8-bit data to slave
                    i2c_txNackBlocking(reg);        // send nack bit
                }
        }
        i2c_txByte(reg, data[length-1]);            // send last 8-bit data to slave
        i2c_txNackBlocking(reg);                    // send nack bit
        i2c_masterStopBlocking(reg);                // send stop sequence
    }


    /*
     *  I2C Read data without register address
     */
    static void i2c_readData(u32 reg, u8 slaveAddr, u8 *data , u32 length){
        i2c_masterStartBlocking(reg);               // Send start sequence
        i2c_txByte(reg, slaveAddr|I2C_READ);        // write device address byt ewith read bit
        i2c_txNackBlocking(reg);                    // send nack bit
        if(length > 1){
            for(int i = 0; i < length - 1; i++){
                i2c_txByte(reg, 0xFF);              // send 0xFF (Release SDA line) to the slave while generate 8-bit SCL pulses
                i2c_txAckBlocking(reg);             // send ack bit to ask slave to continue send the next byte
                data[i] = i2c_rxData(reg);          // read the data from rx data register and place it into data array
            }
        }
        i2c_txByte(reg, 0xFF);                      // send 0xFF (Release SDA line) to the slave while generate 8-bit SCL pulses
        i2c_txNackBlocking(reg);                    // send nack bit
        data[length-1] = i2c_rxData(reg);           // read the data from rx data register and place it into last data array
        i2c_masterStopBlocking(reg);                // send stop sequence
    }
