// The MIT License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
#include <stdbool.h>

#include "bsp.h"
#include "test.h"

// Please copy sd_ctrl.c and sd_ctrl.h from the links below to the "/sd_ctrl" folder.
// https://github.com/Efinix-Inc/xyloni/tree/master/design/soc_Opal_t8/soc_Opal_sw_t8/software/standalone/Xyloni_SelfTest/src
#if __has_include("sd_ctrl/sd_ctrl.h") & __has_include("sd_ctrl/sd_ctrl.c")
#define SPI1	SYSTEM_SPI_0_IO_CTRL

#include "sd_ctrl/sd_ctrl.h"
#include "sd_ctrl/sd_ctrl.c"

#include "../PrimroseSoC/src/lib/apb3.h"

#define SD_DET_SAMPLING_US		1000
#define SD_DET_SAMPLING_TIMES	10

bool sdTest(test_state_t test_state)
{
	static int ct = 0;

	if (apb3_readStatus(APB0) & STAT_SD_DET_N) {
		ct = 0;
	} else {
		if (ct < SD_DET_SAMPLING_TIMES) {
			ct++;
    		if (ct >= SD_DET_SAMPLING_TIMES) {
    			Spi_Read_SDCard_Info();
    		}
		}
	}
	bsp_uDelay(SD_DET_SAMPLING_US);

	return true;
}

void sdTestInit(void)
{
	spi_init();
}

#else
bool sdTest(test_state_t test_state)
{
	if (test_state == test_state_start) {
		bsp_printf("\n\nsd_ctrl.c and/or sd_ctrl.h were not found\n\n");
	}
	return false;
}

void sdTestInit(void)
{
	;
}
#endif
