// The MIT License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
#include <stdbool.h>

#include <bsp.h>

#include "uart.h"
#include "test.h"
#include "gpio_test.h"
#include "apb3_test.h"
#include "spi_test.h"
#include "sd_test.h"
#include "uart_test.h"

static volatile int testReqNum = -1;
static int testNum = -1;

static bool testMenu(test_state_t test_state);

static const struct {
	char		c;
	bool		(*p)(test_state_t);
	const char*	s;
} testProcTbl[] = {
	{'1', apb3_dispVerDateTest,		"APB3 version and date display"},
	{'2', apb3_sledTest,			"APB3 serial LED test"},
	{'3', gpioTest,					"GPIO test"},
	{'4', spi_adcDacTest,			"SPI ADC/DAC test"},
	{'5', spi_currentMonitorTest,	"SPI Current monitor test"},
	{'6', spi_flashAccessTest,		"SPI Flash access test"},
	{'7', apb3_LvdsLoopbackTest,	"APB3 LVDS loop-back test"},
	{'8', sdTest,					"SD test"},
	{'9', uartTest,					"UART test"},
	{'0', testMenu, 				"Test Menu"},
};

static bool testMenu(test_state_t test_state)
{
	if (test_state == test_state_start) {
 		for (int i = 0; i < sizeof(testProcTbl)/sizeof(testProcTbl[0]); i++) {
 			bsp_putChar(testProcTbl[i].c);
			bsp_printf(".%s\n", testProcTbl[i].s);
 		}
		bsp_printf("\nPlease hit key in the number of the test...\n");
	}
	return false;
}

void testRequest(char c)
{
	for (int i = 0; i < sizeof(testProcTbl)/sizeof(testProcTbl[0]); i++) {
		if (testProcTbl[i].c == c) {
			testReqNum = i;
			break;
		}
	}
}

static bool testExec(int num, test_state_t test_state)
{
	bool retval = false;

	if (num != -1) {
		if (test_state == test_state_start) {
	 		bsp_printf("\n\n===%s===\n\n", testProcTbl[num].s);
		}
		retval = testProcTbl[num].p(test_state);
	}

	return retval;
}

void test(void)
{
	if ((testNum != testReqNum) && (testReqNum != -1)) {
		testExec(testNum, test_state_end);
		testNum = testReqNum;
		testReqNum = -1;
		if (!testExec(testNum, test_state_start)) {
			testNum = -1;
		}
	} else {
		if (!testExec(testNum, test_state_exec)) {
			testNum = -1;
		}
	}
}

void testInit(void)
{
	gpioTestInit();
	spiTestInit();
	sdTestInit();
	uartSetReceiveProc(testRequest);
}
