// The MIT License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the “Software”), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
#include <stdbool.h>

#include <bsp.h>

#include "uart.h"
#include "test.h"
#include "uart_test.h"

#define UART_RX_DUMMY_LOAD_US		10000

static bool exec = false;

static void uartTestReceivedProc(char c)
{
#ifdef UART_RX_DUMMY_LOAD_US
	bsp_uDelay(UART_RX_DUMMY_LOAD_US);
#endif
	if (c == '\x1b') {
		exec = false;
	}
	if (exec) {
		uartSendChar(c);
	}
}

bool uartTest(test_state_t test_state)
{
	if (test_state == test_state_start) {
		bsp_printf("[To exit, press the ESC key.]\n\n");
		exec = true;
		uartSetReceiveProc(uartTestReceivedProc);
	}

	if (!exec) {
		uartSetReceiveProc(testRequest);
		bsp_uDelay(1000);
		bsp_printf("\n\n[UART test exited.]\n\n");
		return false;
	}

	return true;
}
