// The CERN-OHL-P v2 License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// This source describes Open Hardware and is licensed under the CERN-OHL-P v2.
// You may redistribute and modify this documentation and make products using it under the terms of
// the CERN-OHL-P v2 (https:/cern.ch/cern-ohl). This documentation is distributed WITHOUT ANY
// EXPRESS OR IMPLIED WARRANTY, INCLUDING OF MERCHANTABILITY, SATISFACTORY QUALITY AND FITNESS FOR A
// PARTICULAR PURPOSE. Please see the CERN-OHL-P v2 for applicable conditions.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
`timescale 1ps / 1ps

module sim;

	// Parameters
	parameter CLK_CYCLE = 10000;							// 100MHz
	parameter LFSR_WIDTH = 8;
	parameter [LFSR_WIDTH-1:0] POLYNOMIAL = 8'b10111000;	// x^8 + x^6 + x^5 + x^4 + 1
//	parameter [LFSR_WIDTH-1:0] POLYNOMIAL = 8'b10001110;	// x^8 + x^4 + x^3 + x^2 + 1
//	parameter [LFSR_WIDTH-1:0] POLYNOMIAL = 8'b10010101;	// x^8 + x^5 + x^3 + x^1 + 1
//	parameter [LFSR_WIDTH-1:0] POLYNOMIAL = 8'b11000011;	// x^8 + x^7 + x^2 + x^1 + 1
//	parameter [LFSR_WIDTH-1:0] POLYNOMIAL = 8'b11100111;	// x^8 + x^7 + x^6 + x^3 + x^2 + x^1 +1
	parameter [LFSR_WIDTH-1:0] INITIAL_VAL = {LFSR_WIDTH{1'b1}};
	
	// Inputs
	reg CLOCK = 1'b0;
	reg RESET = 1'b0;
	reg IN_CLR = 1'b0;
	reg IN_ENA = 1'b0;

	reg duplicate = 1'b0;
	reg [2**LFSR_WIDTH-1:0] flags = {(2**LFSR_WIDTH){1'b0}};

	// Outputs
	wire [LFSR_WIDTH-1:0] OUT_DATA;
	wire OUT_ENA;
	wire OUT_CYCLE;

	lfsr # (
		.LFSR_WIDTH(LFSR_WIDTH),
		.POLYNOMIAL(POLYNOMIAL),
		.INITIAL_VAL(INITIAL_VAL)
	) dut1 (
		.CLOCK(CLOCK),
		.RESET(RESET),
		.IN_CLR(IN_CLR),
		.IN_ENA(IN_ENA),
		.OUT_DATA(OUT_DATA),
		.OUT_ENA(OUT_ENA),
		.OUT_CYCLE(OUT_CYCLE)
	);

	initial begin
		$dumpfile("outflow/lfsr.vcd");
		$dumpvars(0, sim);
	end

	always #(CLK_CYCLE / 2) CLOCK <= ~CLOCK;

	task task_delay (
			input integer value
		);
		begin
			repeat (value)
				@(posedge CLOCK);
			#1;
		end
	endtask

	task task_lfsr_run (
			input integer lfsr_num
		);
		begin
			flags = {(2**LFSR_WIDTH){1'b0}};
			IN_ENA = 1'b1;
			for (integer i = 0; i < lfsr_num; i = i + 1) begin
				if (flags[OUT_DATA])
					duplicate = 1'b1;
				else begin
					flags[OUT_DATA] = 1'b1;
				end
				task_delay(1);
				duplicate = 1'b0;
			end
			IN_ENA = 1'b0;
		end
	endtask

	initial begin
		#(CLK_CYCLE * 5.25);
		RESET = 1'b1;

		task_delay(10);

		task_lfsr_run(2**LFSR_WIDTH);		task_delay(10);
		task_lfsr_run(2**(LFSR_WIDTH-1));	task_delay(10);
		IN_CLR = 1'b1;						task_delay(10);
		IN_CLR = 1'b0;						task_delay(10);
		task_lfsr_run(2**LFSR_WIDTH*2);		task_delay(10);

		$finish;
	end

endmodule

