// The CERN-OHL-P v2 License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// This source describes Open Hardware and is licensed under the CERN-OHL-P v2.
// You may redistribute and modify this documentation and make products using it under the terms of
// the CERN-OHL-P v2 (https:/cern.ch/cern-ohl). This documentation is distributed WITHOUT ANY
// EXPRESS OR IMPLIED WARRANTY, INCLUDING OF MERCHANTABILITY, SATISFACTORY QUALITY AND FITNESS FOR A
// PARTICULAR PURPOSE. Please see the CERN-OHL-P v2 for applicable conditions.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
module lvds_test # (
		parameter TEST_NUMS = 5,	// Fixed value
		parameter DATA_WIDTH = 8,	// Fixed value
		parameter LFSR_WIDTH = 8	// Fixed value
	) (
		input TX_CLK,
		input TX_PLL_LOCKED,
		input RX_CLK,
		input RX_PLL_LOCKED,
		input [TEST_NUMS-1:0] IN_RUN,

		output [DATA_WIDTH-1:0] LVDS_TX_DATA0,
		output [DATA_WIDTH-1:0] LVDS_TX_DATA1,
		output [DATA_WIDTH-1:0] LVDS_TX_DATA2,
		output [DATA_WIDTH-1:0] LVDS_TX_DATA3,
//		output [DATA_WIDTH-1:0] LVDS_TX_DATA4,
		output LVDS_TX_DATA4,		                // Disable serialization
//		output [DATA_WIDTH-1:0] LVDS_TX_DATA5,      // Assign to TX FAST CLOCK

		input [DATA_WIDTH-1:0] LVDS_RX_DATA0,
		input [DATA_WIDTH-1:0] LVDS_RX_DATA1,
		input [DATA_WIDTH-1:0] LVDS_RX_DATA2,
		input [DATA_WIDTH-1:0] LVDS_RX_DATA3,
		input LVDS_RX_DATA4,                        // Disable serialization
		input LVDS_RX_DATA5,                        // Disable serialization and invert polarity

		output [TEST_NUMS-1:0] OUT_RX_EXISTS,
		output [TEST_NUMS-1:0] OUT_ERROR
	);

	wire [DATA_WIDTH-1:0] LVDS_TX_DATA[0:TEST_NUMS-1];

	assign LVDS_TX_DATA0 = LVDS_TX_DATA[0];
	assign LVDS_TX_DATA1 = LVDS_TX_DATA[1];
	assign LVDS_TX_DATA2 = LVDS_TX_DATA[2];
	assign LVDS_TX_DATA3 = LVDS_TX_DATA[3];
	assign LVDS_TX_DATA4 = LVDS_TX_DATA[4][0];

	generate
		for (genvar i = 0; i < TEST_NUMS; i = i + 1) begin
			if (i != TEST_NUMS - 1) begin
				lvds_loopback # (
					.DATA_WIDTH(DATA_WIDTH),
					.LFSR_WIDTH(LFSR_WIDTH),
					.POLYNOMIAL((i == 0)? 8'b10111000 :	// x^8 + x^6 + x^5 + x^4 + 1
								(i == 1)? 8'b10001110 :	// x^8 + x^4 + x^3 + x^2 + 1
								(i == 2)? 8'b10010101 : // x^8 + x^5 + x^3 + x^1 + 1	
								(i == 3)? 8'b11000011 : // x^8 + x^7 + x^2 + x^1 + 1
										  8'b00000000),
					.INITIAL_VAL({LFSR_WIDTH{1'b1}}),
					.RX_DATA_INV(0)
				) lvds_loopback_u (
					.TX_CLK(TX_CLK),
					.TX_PLL_LOCKED(TX_PLL_LOCKED),
					.RX_CLK(RX_CLK),
					.RX_PLL_LOCKED(RX_PLL_LOCKED),
					.IN_RUN(IN_RUN[i]),
					.LVDS_RX_DATA((i == 0)? LVDS_RX_DATA0 :
								  (i == 1)? LVDS_RX_DATA1 :
								  (i == 2)? LVDS_RX_DATA2 :
								  (i == 3)? LVDS_RX_DATA3 :
											{DATA_WIDTH{1'b0}}),
					.LVDS_RX_INV_DATA({DATA_WIDTH{1'b0}}),
					.LVDS_TX_DATA(LVDS_TX_DATA[i]),
					.OUT_RX_EXISTS(OUT_RX_EXISTS[i]),
					.OUT_ERROR(OUT_ERROR[i])
				);
			end
			else begin
				lvds_loopback # (
					.DATA_WIDTH(1),
					.LFSR_WIDTH(LFSR_WIDTH),
					.POLYNOMIAL(8'b11100111), // x^8 + x^7 + x^6 + x^3 + x^2 + x^1 +1
					.INITIAL_VAL({LFSR_WIDTH{1'b1}}),
					.RX_DATA_INV(1)
				) lvds_loopback_u (
					.TX_CLK(TX_CLK),
					.TX_PLL_LOCKED(TX_PLL_LOCKED),
					.RX_CLK(~RX_CLK),
					.RX_PLL_LOCKED(RX_PLL_LOCKED),
					.IN_RUN(IN_RUN[i]),
					.LVDS_RX_DATA(LVDS_RX_DATA4),
					.LVDS_RX_INV_DATA(LVDS_RX_DATA5),
					.LVDS_TX_DATA(LVDS_TX_DATA[i][0]),
					.OUT_RX_EXISTS(OUT_RX_EXISTS[i]),
					.OUT_ERROR(OUT_ERROR[i])
				);
			end
		end
	endgenerate

endmodule
