// The CERN-OHL-P v2 License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// This source describes Open Hardware and is licensed under the CERN-OHL-P v2.
// You may redistribute and modify this documentation and make products using it under the terms of
// the CERN-OHL-P v2 (https:/cern.ch/cern-ohl). This documentation is distributed WITHOUT ANY
// EXPRESS OR IMPLIED WARRANTY, INCLUDING OF MERCHANTABILITY, SATISFACTORY QUALITY AND FITNESS FOR A
// PARTICULAR PURPOSE. Please see the CERN-OHL-P v2 for applicable conditions.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
module s_ff # (
		parameter CLOCK_POL = 1,
		parameter RESET_POL = 0,
		parameter ARESET = 1,
		parameter DATA_WIDTH = 8,
		parameter SHIFT_NUMS = 2
	) (
		input CLOCK,
		input RESET,
		input [DATA_WIDTH-1:0] IN_DATA,
		output [DATA_WIDTH-1:0] OUT_DATA
	);

	wire [DATA_WIDTH-1:0] data [SHIFT_NUMS:0];

	assign data[0] = IN_DATA;
	assign OUT_DATA = data[SHIFT_NUMS];

	generate
		for (genvar i = 0; i < SHIFT_NUMS; i = i + 1) begin //: gen_d_ff
			d_ff #(
				.CLOCK_POL(CLOCK_POL),
				.RESET_POL(RESET_POL),
				.ARESET(ARESET),
				.DATA_WIDTH(DATA_WIDTH)
			) d_ff_u (
				.CLOCK(CLOCK),
				.RESET(RESET),
				.IN_DATA(data[i]),
				.OUT_DATA(data[i+1])
			);
		end
	endgenerate
	
endmodule