// The CERN-OHL-P v2 License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// This source describes Open Hardware and is licensed under the CERN-OHL-P v2.
// You may redistribute and modify this documentation and make products using it under the terms of
// the CERN-OHL-P v2 (https:/cern.ch/cern-ohl). This documentation is distributed WITHOUT ANY
// EXPRESS OR IMPLIED WARRANTY, INCLUDING OF MERCHANTABILITY, SATISFACTORY QUALITY AND FITNESS FOR A
// PARTICULAR PURPOSE. Please see the CERN-OHL-P v2 for applicable conditions.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
`timescale 1ps / 1ps

module sim;

	// Parameters
	parameter CLK_CYCLE = 10000;	// 100MHz
	parameter DATA_WIDTH = 8;
	parameter SHIFT_NUMS = 3;

	// Inputs
	reg CLOCK = 1'b0;
	reg RESET_N = 1'b0;
	reg [DATA_WIDTH-1:0]	IN_DATA = {DATA_WIDTH{1'b0}};

	// Outputs
	wire [DATA_WIDTH-1:0]	OUT_DATA1;
	wire [DATA_WIDTH-1:0]	OUT_DATA2;
	wire [DATA_WIDTH-1:0]	OUT_DATA3;
	wire [DATA_WIDTH-1:0]	OUT_DATA4;

	s_ff #(
		.CLOCK_POL(1),
		.RESET_POL(0),
		.ARESET(0),
		.DATA_WIDTH(DATA_WIDTH),
		.SHIFT_NUMS(SHIFT_NUMS)
	) dut1 (
		.CLOCK(CLOCK), 
		.RESET(RESET_N), 
		.IN_DATA(IN_DATA),
		.OUT_DATA(OUT_DATA1)
	);

	s_ff #(
		.CLOCK_POL(0),
		.RESET_POL(0),
		.ARESET(0),
		.DATA_WIDTH(DATA_WIDTH),
		.SHIFT_NUMS(SHIFT_NUMS)
	) dut2 (
		.CLOCK(~CLOCK), 
		.RESET(RESET_N), 
		.IN_DATA(IN_DATA),
		.OUT_DATA(OUT_DATA2)
	);

	s_ff #(
		.CLOCK_POL(1),
		.RESET_POL(1),
		.ARESET(0),
		.DATA_WIDTH(DATA_WIDTH),
		.SHIFT_NUMS(SHIFT_NUMS)
	) dut3 (
		.CLOCK(CLOCK), 
		.RESET(~RESET_N), 
		.IN_DATA(IN_DATA),
		.OUT_DATA(OUT_DATA3)
	);

	s_ff #(
		.CLOCK_POL(1),
		.RESET_POL(0),
		.ARESET(1),
		.DATA_WIDTH(DATA_WIDTH),
		.SHIFT_NUMS(SHIFT_NUMS)
	) dut4 (
		.CLOCK(CLOCK), 
		.RESET(RESET_N), 
		.IN_DATA(IN_DATA),
		.OUT_DATA(OUT_DATA4)
	);

	initial begin
		$dumpfile("outflow/s_ff.vcd");
		$dumpvars(0, sim);
	end

	always #(CLK_CYCLE / 2) CLOCK <= ~CLOCK;

	task inputData (
			input [DATA_WIDTH-1:0]	data
		);
		begin
			@(posedge CLOCK)
			#1	IN_DATA <= data;
		end
	endtask

	initial begin

		#(CLK_CYCLE * 5.25);
		RESET_N = 1'b1;		
		#(CLK_CYCLE * 5);

		for (integer i = 0; i < DATA_WIDTH; i = i + 1)
			inputData({{DATA_WIDTH-1{1'b0}},1'b1} << i);

		#(CLK_CYCLE * 5.25);
		RESET_N = 1'b0;
		#(CLK_CYCLE * 5);

		$finish;
	end
	
endmodule

