// The CERN-OHL-P v2 License
//
// Copyright (c) Since 2024 RIGHTxLIGHT Co., Ltd.
//
// This source describes Open Hardware and is licensed under the CERN-OHL-P v2.
// You may redistribute and modify this documentation and make products using it under the terms of
// the CERN-OHL-P v2 (https:/cern.ch/cern-ohl). This documentation is distributed WITHOUT ANY
// EXPRESS OR IMPLIED WARRANTY, INCLUDING OF MERCHANTABILITY, SATISFACTORY QUALITY AND FITNESS FOR A
// PARTICULAR PURPOSE. Please see the CERN-OHL-P v2 for applicable conditions.
//
// Revisions:
// Rev.1.0		2024/04/16		Initial release
//
// -------------------------------------------------------------------------------------------------
`timescale 1ps / 1ps

module sim;

	// Parameters
	parameter CLK_CYCLE = 10000;							// 100MHz
	parameter T_WIDTH = 18;
	parameter T0H_WIDTH = 3;
	parameter T1H_WIDTH = 10;
	parameter T_RESET_WIDTH = 512;
	parameter LED_NUMS = 3;
	parameter ADDR_WIDTH = 2;
	parameter DATA_WIDTH = 24;								// 8bits * 3colors

	// Inputs
	reg CLOCK = 1'b0;
	reg RESET = 1'b0;
	reg [DATA_WIDTH-1:0] IN_DATA = 0;
	reg [ADDR_WIDTH-1:0] IN_ADDR = 0;
	reg IN_WRITE = 1'b0;
	reg IN_LED_SET = 1'b0;

	// Outputs
	wire OUT_BUSY;
	wire OUT_LED_DATA;

	sled_drv # (
		.T_WIDTH(T_WIDTH),
		.T0H_WIDTH(T0H_WIDTH),
		.T1H_WIDTH(T1H_WIDTH),
		.T_RESET_WIDTH(T_RESET_WIDTH),
		.LED_NUMS(LED_NUMS),
		.DATA_WIDTH(DATA_WIDTH),
		.ADDR_WIDTH(ADDR_WIDTH)
	) dut1 (
		.CLOCK(CLOCK),
		.RESET(RESET),
		.IN_DATA(IN_DATA),
		.IN_ADDR(IN_ADDR),
		.IN_WRITE(IN_WRITE),
		.IN_LED_SET(IN_LED_SET),
		.OUT_BUSY(OUT_BUSY),
		.OUT_LED_DATA(OUT_LED_DATA)
	);

	initial begin
		$dumpfile("outflow/sled_drv.vcd");
		$dumpvars(0, sim);
	end

	always #(CLK_CYCLE / 2) CLOCK <= ~CLOCK;

	task task_delay (
			input integer value
		);
		begin
			repeat (value)
				@(posedge CLOCK);
			#1;
		end
	endtask

	task writeData (
			input [DATA_WIDTH-1:0]	data,
			input [DATA_WIDTH-1:0]	addr,
			input wr,
			input set
		);
		begin
			@(posedge CLOCK)
			#1	IN_DATA <= data;
				IN_ADDR <= addr;
				IN_WRITE <= wr;
				IN_LED_SET <= set;
			@(posedge CLOCK)
			@(posedge CLOCK)
			#1	IN_WRITE <= 0;
				IN_LED_SET <= 0;
		end
	endtask

	initial begin
		#(CLK_CYCLE * 5.25);
		RESET = 1'b1;		

		task_delay(10);
		writeData(24'h123456, 0, 1, 0);
		task_delay(10);
		writeData(24'h789ABC, 1, 1, 0);
		task_delay(10);
		writeData(24'hDEF012, 2, 1, 1);
		task_delay(3000);

		writeData(24'h0000FF, 0, 1, 0);
		task_delay(10);
		writeData(24'h00FF00, 1, 1, 0);
		task_delay(10);
		writeData(24'hFF0000, 2, 1, 1);
		task_delay(3000);

		$finish;
	end
      
endmodule

